from enum import StrEnum


class MinecraftPotionEffectTypes(StrEnum):
    Awkward = "minecraft:awkward"
    FireResistance = "minecraft:fire_resistance"
    Harming = "minecraft:harming"
    Healing = "minecraft:healing"
    Infested = "minecraft:infested"
    Invisibility = "minecraft:invisibility"
    Leaping = "minecraft:leaping"
    LongFireResistance = "minecraft:long_fire_resistance"
    LongInvisibility = "minecraft:long_invisibility"
    LongLeaping = "minecraft:long_leaping"
    LongMundane = "minecraft:long_mundane"
    LongNightvision = "minecraft:long_nightvision"
    LongPoison = "minecraft:long_poison"
    LongRegeneration = "minecraft:long_regeneration"
    LongSlowFalling = "minecraft:long_slow_falling"
    LongSlowness = "minecraft:long_slowness"
    LongStrength = "minecraft:long_strength"
    LongSwiftness = "minecraft:long_swiftness"
    LongTurtleMaster = "minecraft:long_turtle_master"
    LongWaterBreathing = "minecraft:long_water_breathing"
    LongWeakness = "minecraft:long_weakness"
    Mundane = "minecraft:mundane"
    Nightvision = "minecraft:nightvision"
    Oozing = "minecraft:oozing"
    Poison = "minecraft:poison"
    Regeneration = "minecraft:regeneration"
    SlowFalling = "minecraft:slow_falling"
    Slowness = "minecraft:slowness"
    Strength = "minecraft:strength"
    StrongHarming = "minecraft:strong_harming"
    StrongHealing = "minecraft:strong_healing"
    StrongLeaping = "minecraft:strong_leaping"
    StrongPoison = "minecraft:strong_poison"
    StrongRegeneration = "minecraft:strong_regeneration"
    StrongSlowness = "minecraft:strong_slowness"
    StrongStrength = "minecraft:strong_strength"
    StrongSwiftness = "minecraft:strong_swiftness"
    StrongTurtleMaster = "minecraft:strong_turtle_master"
    Swiftness = "minecraft:swiftness"
    Thick = "minecraft:thick"
    TurtleMaster = "minecraft:turtle_master"
    Water = "minecraft:water"
    WaterBreathing = "minecraft:water_breathing"
    Weakness = "minecraft:weakness"
    Weaving = "minecraft:weaving"
    WindCharged = "minecraft:wind_charged"
    Wither = "minecraft:wither"


class MinecraftPotionLiquidTypes(StrEnum):
    Consume = "Consume"
    ThrownLingering = "ThrownLingering"
    ThrownSplash = "ThrownSplash"


class MinecraftEffects(StrEnum):
    """
    Enumeration for the different types of effects in the game.
    """

    Absorption = "minecraft:absorption"
    BadOmen = "minecraft:bad_omen"
    Blindness = "minecraft:blindness"
    BreathOfTheNautilus = "minecraft:breath_of_the_nautilus"
    ConduitPower = "minecraft:conduit_power"
    Darkness = "minecraft:darkness"
    FatalPoison = "minecraft:fatal_poison"
    FireResistance = "minecraft:fire_resistance"
    Haste = "minecraft:haste"
    HealthBoost = "minecraft:health_boost"
    Hunger = "minecraft:hunger"
    Infested = "minecraft:infested"
    InstantDamage = "minecraft:instant_damage"
    InstantHealth = "minecraft:instant_health"
    Invisibility = "minecraft:invisibility"
    JumpBoost = "minecraft:jump_boost"
    Levitation = "minecraft:levitation"
    MiningFatigue = "minecraft:mining_fatigue"
    Nausea = "minecraft:nausea"
    NightVision = "minecraft:night_vision"
    Oozing = "minecraft:oozing"
    Poison = "minecraft:poison"
    RaidOmen = "minecraft:raid_omen"
    Regeneration = "minecraft:regeneration"
    Resistance = "minecraft:resistance"
    Saturation = "minecraft:saturation"
    SlowFalling = "minecraft:slow_falling"
    Slowness = "minecraft:slowness"
    Speed = "minecraft:speed"
    Strength = "minecraft:strength"
    TrialOmen = "minecraft:trial_omen"
    VillageHero = "minecraft:village_hero"
    WaterBreathing = "minecraft:water_breathing"
    Weakness = "minecraft:weakness"
    Weaving = "minecraft:weaving"
    WindCharged = "minecraft:wind_charged"
    Wither = "minecraft:wither"
